include<metric.scad>;
include<tile.scad>;
include<util.scad>;

LX15D_dim = [ 44.17, 22.95, 25.5 ];
LX15D_middle_t = 1.5;  // height of the middle space
LX15D_middle_a = 3.0;  // side space
LX15D_axis_offset = 10.66;
LX15D_horn_d = 20;
LX15D_horn_w = 38.6;
LX15D_screw_x = 41.2;
LX15D_screw_y = 18;
LX15D_horn_screw_d = 3;
LX15D_screw_clearance_d = 7;
LX15D_body_screw_d = 2;
LX15D_horn_clearance_d = 8;
LX15D_clearance_r = sqrt(pow(LX15D_dim[1] / 2, 2) + pow(LX15D_axis_offset, 2));

module LX15D() finalize("gray") difference() {
    dim = LX15D_dim;
    union() {
        translate([ dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
            cube(dim, center = true);
            difference() {
                cube(
                    dim +
                        [
                            -LX15D_middle_a, -LX15D_middle_a, LX15D_middle_t * 2
                        ],
                    center = true);
                for (i = [ -0.5, 0.5 ], j = [ -0.5, 0.5 ])
                    translate([ dim[0] * i, dim[1] * j, 0 ])
                        cylinder(d = 13, h = 100, center = true);
            }
        }

        cylinder(d = LX15D_horn_d, h = LX15D_horn_w, center = true);
    }
    for (i = [ -1, 1 ]) {
        LX15D_horn_screw_pos() translate([ 0, 0, i * LX15D_horn_w / 2 ])
            cylinder(d = LX15D_horn_screw_d, h = 10, center = true);
    }
    for (m = [ 0, 1 ])
        mirror([ 0, 0, m ]) LX15D_bottom_screw_pos()
            cylinder(d = LX15D_body_screw_d, h = 4, center = true);
}

module LX15D_collision() {
    dim = LX15D_dim;

    translate([ dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
        cube(dim + [ 0, 0, LX15D_middle_t * 2 ], center = true);
    }

    cube([ LX15D_horn_d, LX15D_horn_d, LX15D_horn_w ], center = true);
}

module LX15D_clearance() {
    cylinder(r = LX15D_clearance_r, h = LX15D_horn_w, center = true);
}

module LX15D_screw_plane() { translate([ 0, 0, LX15D_dim[2] / 2 ]) children(); }

module LX15D_bottom_screw_pos()
    translate([ LX15D_dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
    for (y = [ -0.5, 0.5 ])
        translate([ LX15D_screw_x / 2, y * LX15D_screw_y, LX15D_dim[2] / 2 ])
            children();
}

module LX15D_horn_screw_pos() {
    for (a = [0:90:360])
        rotate([ 0, 0, a ]) translate([ 14 / 2, 0, 0 ]) children();
}
module LX15D_side_plate(screw_d, holes, t) difference() {
    union() {
        LX15D_bottom_screw_pos() hull() {
            cylinder(d = LX15D_screw_clearance_d, h = t);
            translate([ t * 2, 0, 0 ])
                cylinder(d = LX15D_screw_clearance_d, h = t);
        }
        hull() for (coord = holes) {
            translate(coord + [ 0, 0, LX15D_dim[2] / 2 ])
                cylinder(d = screw_d + t * 2, h = t);
        }
    }
    for (coord = holes)
        translate(coord) cylinder(d = screw_d, h = 100, center = true);
    LX15D_bottom_screw_pos()
        cylinder(d = LX15D_body_screw_d, h = 100, center = true);
}
LX15D_U_h = 6;
LX15D_U_c = 0.2;
LX15D_U_offset = [ 40, 0, 0 ];
LX15D_U_screw_y = 16;
module LX15D_U_body(class) difference() {
    t = 2;
    translate([ -LX15D_U_h / 2, 0, 0 ])
        cube([ LX15D_U_h, Ta(T24), LX15D_dim[2] ], center = true);

    rotate([ 0, -90, 0 ]) tile_neg(T24, LX15D_U_h, centered_hole = true);
    rotate([ 0, 90, 0 ])
        cylinder(d = Ta(T24) / 2, h = 100, center = true, $fn = 8);

    for (i = [ -0.5, 0.5 ], m = [ 0, 1 ])
        mirror([ 0, 0, m ]) translate(
            [ -LX15D_U_h / 2, i * LX15D_U_screw_y, -LX15D_dim[2] / 2 - t ]) {
            Mscrew_hole(M2L8C);
            translate([ 0, 0, 2 * t ]) Mnut_sidehole(M2N, l = 10);
        }
}
module LX15D_U_plate(class) difference() {
    t = 2;
    union() {
        translate(LX15D_U_offset -
                  [ LX15D_U_h / 2, 0, LX15D_dim[2] / 2 + t / 2 ])
            cube([ LX15D_U_h, Ta(T24), t ], center = true);
        mirror([ 0, 0, 1 ]) LX15D_bottom_screw_pos()
            hull() for (x = [ 0, LX15D_U_h / 2 ]) translate([ x, 0, 0 ])
                cylinder(d = LX15D_screw_clearance_d, h = t);
    }
    translate(LX15D_U_offset - [ LX15D_U_h / 2, 0, LX15D_dim[2] / 2 + t ]) {
        for (i = [ -0.5, 0.5 ])
            translate([ 0, i * LX15D_U_screw_y, 0 ]) Mscrew_hole(M2L8C);
    }
    LX15D_bottom_screw_pos()
        cylinder(d = LX15D_body_screw_d, h = 100, center = true);
}
module LX15D_U(class, expand = 0) union() {
    t = 2;
    translate(LX15D_U_offset) finalize("blue") LX15D_U_body(class);

    for (m = [ 0, 1 ]) mirror([ 0, 0, m ]) {
            translate([ 0, 0, -expand ]) finalize("yellow")
                LX15D_U_plate(class);

            translate(LX15D_U_offset) for (i = [ -0.5, 0.5 ]) translate([
                -LX15D_U_h / 2, i * LX15D_U_screw_y, -LX15D_dim[2] / 2 - t - 2 *
                expand
            ]) Mscrew(M2L8C);

            LX15D_bottom_screw_pos() translate([ 0, 0, 3 * expand + t ])
                rotate([ 180, 0, 0 ]) Mscrew(M2L8C);

            for (i = [ -0.5, 0.5 ])
                translate(LX15D_U_offset + [
                    expand - LX15D_U_h / 2, i * LX15D_U_screw_y,
                    -LX15D_dim[2] / 2 +
                    t
                ]) Mnut(M2N);
        }
}

LX15D_BU_h = 6;
LX15D_BU_t = 3;
LX15D_BU_screw_a = 10;
LX15D_BU_offset = [ -25, 0, 0 ];
module LX15D_BU_plate() difference() {
    t = LX15D_BU_t;
    h = 5;
    horn_a = LX15D_horn_screw_d + t;
    bu_screw_a = Md(M2L8C) + t;
    translate([ 0, 0, h / 2 ]) hull() {
        LX15D_horn_screw_pos() cube([ horn_a, horn_a, h ], center = true);
        for (i = [ -0.5, 0.5 ])
            translate(LX15D_BU_offset +
                      [ LX15D_BU_h / 2, i * LX15D_BU_screw_a, 0 ])
                cube([ bu_screw_a, bu_screw_a, h ], center = true);
    }
    cylinder(d = LX15D_horn_clearance_d, h = 100, center = true, $fn = 8);
    LX15D_horn_screw_pos() Mscrew_hole(M3L8C);
    for (i = [ -0.5, 0.5 ])
        translate(LX15D_BU_offset + [ LX15D_BU_h / 2, i * LX15D_BU_screw_a, t ])
            rotate([ 180, 0, 0 ]) Mscrew_hole(M2L8C);
}
module LX15D_BU_body() translate(LX15D_BU_offset + [ LX15D_BU_h / 2, 0, 0 ])
    difference() {
    t = LX15D_BU_t;
    dim = [ LX15D_BU_h, LX15D_BU_screw_a + t * 2, LX15D_horn_w ];
    union() {
        rotate([ 0, 90, 0 ]) tile_pos(T24, LX15D_BU_h, center = true);
        cube(dim, center = true);
    }
    rotate([ 0, 90, 0 ])
        tile_neg(T24, LX15D_BU_h, center = true);

    for (j = [ -1, 1 ], i = [ -0.5, 0.5 ])
        translate([ 0, i * LX15D_BU_screw_a, j * dim[2] / 2 ])
            rotate([ j * 90 + 90, 0, 0 ]) {
            Mscrew_hole(M2L8C);
            translate([ 0, 0, t ]) Mnut_sidehole(M2N, l = 10);
        }
}

module LX15D_BU(expand = 0) {
    t = 3;
    plate_h = 5;
    finalize("yellow") LX15D_BU_body();
    for (m = [ 0, 1 ])
        mirror([ 0, 0, m ]) translate([ 0, 0, LX15D_horn_w / 2 + expand ]) {
            finalize("blue") LX15D_BU_plate();

            for (i = [ -0.5, 0.5 ])
                translate(LX15D_BU_offset +
                          [ LX15D_BU_h / 2, i * LX15D_BU_screw_a, t + expand ])
                    rotate([ 180, 0, 0 ]) Mscrew(M2L8C);

            LX15D_horn_screw_pos() translate([ 0, 0, plate_h + expand ])
                rotate([ 180, 0, 0 ]) Mscrew(M3L8C);
        }
    for (j = [ -1, 1 ], i = [ -0.5, 0.5 ])
        translate(LX15D_BU_offset + [
            LX15D_BU_h / 2 + expand, i * LX15D_BU_screw_a, j * LX15D_horn_w / 2
        ]) rotate([ j * 90 + 90, 0, 0 ]) {
            translate([ 0, 0, t ]) Mnut(M2N, l = 10);
        }
}

render_part = "nothing";

if (render_part == "LX15D_BU_plate") {
    $fn = 128;
    LX15D_BU_plate();
} else if (render_part == "LX15D_BU_body") {
    $fn = 128;
    LX15D_BU_body();
} else if (render_part == "LX15D_U_plate") {
    $fn = 128;
    LX15D_U_plate();
} else if (render_part == "LX15D_U_body") {
    $fn = 128;
    LX15D_U_body();
}
