include<camera.scad>;
include<metric.scad>;
include<util.scad>;
include<tile.scad>;
include<LX15D.scad>;
include<bat.scad>;

screw_d = 3;
t = 3;
middle_t = 10;

xplate_h = Ta(T24) / 2;
xplate_tile_h = 6;
xplate_w = Ta(T24) * 2;
xplate_l = Ta(T24) * 7;

module body_collision(positions) {
    hull() for (pos = positions) {
        translate(pos)
            cube([ LX15D_dim[1], LX15D_dim[1], LX15D_horn_w ], center = true);
    }
    cube([ xplate_l, xplate_w, xplate_h * 2 + xplate_tile_h * 2 ],
         center = true);
}

module spine() rotate([ 0, 90, 0 ]) {
    for (i = [-1.5:1.5])
        translate([ 0, i * Ta(T24), 0 ])
            tile(T24, middle_t, center = true, centered_hole = true);
}

module xplate_center() difference() {
    r = 2;
    r_sqrd = r / sqrt(2);
    union() {
        for (i = [ -1, 1 ])
            translate([ i * (middle_t + t) / 2, 0, xplate_h / 2 ])
                cube([ t, xplate_w, xplate_h ], center = true);

        translate([ 0, 0, xplate_h + xplate_tile_h / 2 - r / 2 ]) cornered_cube(
            [ Ta(T24), xplate_w, xplate_tile_h + r ], Tcorner(T24));
    }

    for (i = [ -1, 1 ]) {
        hull() for (j = [ 0, i * 1 ])
            translate([ i * (middle_t / 2 + t + r) + j * 10, 0, xplate_h - r ])
                rotate([ 90, 0, 0 ]) cylinder(r = r, h = 100, center = true);
        translate([ i * (middle_t / 2 - r_sqrd), 0, xplate_h - r_sqrd ])
            rotate([ 90, 0, 0 ]) cylinder(r = r, h = 100, center = true);
    }

    for (i = [ -1, 1 ]) translate([ 0, i * Ta(T24) / 2, xplate_h ]) {
            tile_center_hole(T24, 100);
            translate([ 0, 0, Ml(M2L8C) ]) rotate([ 180, 0, 0 ])
                tile_screw_pos(T24) Mscrew_hole(M2L8C);
        }

    cube([ middle_t, 1000, Ta(T24) ], center = true);
    translate([ 0, 0, -50 + 1 ]) cube(100, center = true);

    for (i = [ -1, 1 ], j = [ -0.5, 0.5 ])
        translate([ 0, j * Ta(T24), 0 ]) rotate([ 0, i * 90, 0 ])
            tile_screw_pos(T24) cylinder(d = screw_d, h = 100, center = true);
}

module top_xplate() {
    xplate_center();
    for (j = [ -0.5, 0.5 ], k = [ -3, -2, -1, 1, 2, 3 ])
        translate([ k * Ta(T24), j * Ta(T24), xplate_h ])
            tile(T24, xplate_tile_h);
}
module bottom_xplate() {
    rotate([ 180, 0, 0 ]) xplate_center();
    translate([ 0, 0, -xplate_h - xplate_tile_h ]) {
        for (j = [ -0.5, 0.5 ], k = [ -1, 1 ])
            translate([ k * Ta(T24), j * Ta(T24), 0 ]) tile(T24, xplate_tile_h);
        for (k = [ -3, -2, 2, 3 ])
            translate([ k * Ta(T24), 0, 0 ]) tile(T24, xplate_tile_h);
    }
}

module body(positions, expand = 0) {
    $fn = 32;
    for (m = [ 0, 1 ])
        mirror([ m, 0, 0 ]) translate([ Ta(T24) * 2, 0, 0 ]) rotate([ 0, 0, 0 ])
            bat_set();

    for (pos = positions)
        translate(pos) rotate([ 0, 0, pos[0] > 0 ? 180 : 0 ]) {
            translate([ -4 * expand, 0, 0 ]) LX15D();
            translate([ -2 * expand, 0, 0 ]) LX15D_U(expand = expand);
        }
    union() {
        translate([ 0, 0, 3 * expand ]) finalize("green") top_xplate();
        translate([ 0, 0, -3 * expand ]) finalize("green") bottom_xplate();

        finalize("white") spine();
    }
    translate([
        -2.5 * Ta(T24), 0, xplate_h + xplate_tile_h + D415_dim[2] / 2 + 30
    ]) {
        D415();
        D415_fov();
    }
}

// top_xplate();
$fs=0.5;
$fa=0.5;
bottom_xplate();
//body(positions =
//         [
//             [ 45.0, -36.0, 0.0 ], [ -45.0, -36.0, 0.0 ], [ 45.0, 36.0, 0.0 ],
//             [ -45.0, 36.0, 0.0 ]
//         ],
//     expand = 0);
