
//    0  1      2      3  4       5       6
//    d, nut_d, nut_h, l, head_d, head_h, type
function M2gen(l, type, head_h = 0, head_d = 0) =  //
    [ 2, 4.320, 1.600, l, head_d, head_h, type ];
function M3gen(l, type, head_h = 0, head_d = 0) =  //
    [ 3, 6.300, 2.500, l, head_d, head_h, type ];

function Md(class) = class[0];
function Mnut_d(class) = class[1];
function Mnut_h(class) = class[2];
function Ml(class) = class[3];
function Mhead_d(class) = class[4];
function Mhead_h(class) = class[5];
function Mtype(class) = class[6];

M_FLAT = 0;
M_COLUMN = 1;
M_NUT = 2;

M2L4F = M2gen(4, M_FLAT, head_h = 1.2, head_d = 3.8);
M2L12C = M2gen(12, M_COLUMN, head_h = 2, head_d = 3.98);
M2L8C = M2gen(8, M_COLUMN, head_h = 2, head_d = 3.98);
M2L4C = M2gen(4, M_COLUMN, head_h = 2, head_d = 3.98);
M2N = M2gen(0, M_NUT);

M3L8C = M3gen(8, M_COLUMN, head_h = 3, head_d = 5.22);

module Mscrew_hole(class, hole_offset = 0) {
    assert(Mtype(class) != M_NUT,
           "nut type should not be used with Mscrew_hole");
    cylinder(d = Md(class), h = Ml(class));
    if (Mtype(class) == M_FLAT) {
        translate([ 0, 0, -0.01 ])
            cylinder(d1 = Mhead_d(class), d2 = Md(class), h = Mhead_h(class));
    } else if (Mtype(class) == M_COLUMN) {
        rotate([ 180, 0, 0 ])
            cylinder(d = Mhead_d(class), h = Mhead_h(class) + hole_offset);
    }
}

module Mscrew(class) color("red") render() difference() {
    assert(Mtype(class) != M_NUT, "nut type should not be used with Mscrew");
    union() {
        cylinder(d = Md(class), h = Ml(class));
        if (Mtype(class) == M_FLAT) {
            translate([ 0, 0, -0.01 ]) cylinder(
                d1 = Mhead_d(class), d2 = Md(class), h = Mhead_h(class));
        } else {
            rotate([ 180, 0, 0 ])
                cylinder(d = Mhead_d(class), h = Mhead_h(class));
        }
    }
    rotate([ 180, 0, 0 ]) translate([
        0, 0,
        Mhead_h(class) / 2 + (Mtype(class) == M_FLAT ? -Mhead_h(class) : 0)
    ]) cylinder(d = Md(class), h = 100, $fn = 6);
}

module Mnut(class) color("green") render() difference() {
    assert(Mtype(class) == M_NUT, "only nut types should be used wtih Mnut");
    cylinder(d = Mnut_d(class), h = Mnut_h(class), $fn = 6);
    cylinder(d = Md(class), h = 100, center = true);
}

module Mnut_sidehole(class, l) hull() for (x = [ 0, l ]) translate([ x, 0, 0 ])
    Mnut(class);
