include<metric.scad>;
include<util.scad>;
include<LX15D.scad>;
include<tile.scad>;

module tibia_servo_pos() rotate([ 90, 0, 0 ]) children();

t = 1.5;
screw_d = 2;
screw_e = 3;
screw_t = 1.5;
screw_step = 5;
b = 6;
leg_i = 10;

tile_h = 6;

bot_d = 40;
bot_screw_z = 6;
screw_m = Mhead_d(M2L4C);
mid_h = screw_m / 2 + bot_screw_z;

ball_j_w = 4;
ball_j_d = 7.3;
ball_j_l = 19 - ball_j_d / 2;
ball_angle_limit = 60;

leg_bot_screw_r = 16;

middle_tile_h = 4;

module ball_joint() color("gray") difference() {
    l = 19 - ball_j_d;

    union() {
        hull() {
            translate([ 0, 0, ball_j_d / 2 + l / 2 ])
                cylinder(d = 4.2, h = l / 2);
            translate([ 0, 0, ball_j_d / 2 ])
                cube([ 4.2, ball_j_w / 2, 1 ], center = true);
        }
        sphere(d = ball_j_w);
        hull() {
            translate([ 0, 0, ball_j_d / 2 ])
                cube([ 4.2, ball_j_w / 2, 1 ], center = true);
            rotate([ 90, 0, 0 ])
                cylinder(d = ball_j_d, h = ball_j_w / 2, center = true);
        }
    }
    translate([ 0, 0, ball_j_d / 2 ]) cylinder(d = 2, h = 100);
    rotate([ 90, 0, 0 ]) cylinder(d = 2, h = 100, center = true);
}

function tibia_pyramid_z(tipOffset) =
    (tipOffset[0] - LX15D_U_offset[0]) - ball_j_l - abs(bot_screw_z);

module tibia_pyramid_screw_pos(tipOffset, ignore_middle = false) {
    if (!ignore_middle)
        translate([ 0, 0, tibia_pyramid_z(tipOffset) - t ]) children();
    translate([ 0, 0, Ta(T24) + 2 * middle_tile_h + t ]) {
        rotate([ 180, 0, 0 ]) tile_screw_pos(T24) children();
    }
}

module tibia_pyramid(tipOffset) translate(LX15D_U_offset) rotate([ 0, 90, 0 ])
    difference() {
    hull() {
        translate([ 0, 0, Ta(T24) + 2 * middle_tile_h ]) tile(T24, 1);
        translate([ 0, 0, tibia_pyramid_z(tipOffset) ]) rotate([ 180, 0, 0 ])
            cylinder(d = 2 + t * 2, h = 1);
    }
    tibia_pyramid_screw_pos(tipOffset) Mscrew_hole(M2L8C, hole_offset = 100);
}

function spring_f(i, m, r) =
    [[cos(i - 90), sin(i - 90)], [-sin(i - 90), cos(i - 90)]] *
    [ r + sign(sin(i * m)) * pow(abs(sin(i * m)), 0.4) * 1.5, 0 ];

module leg_bot_base() offset(t) offset(-t) {
    intersection() {
        difference() {
            circle(d = bot_d);
            circle(d = bot_d * 0.55);
        }
        rotate([ 0, 0, -45 ]) translate([ 1, 1, 0 ]) square(bot_d);
    }
}

module screw_mount(center = false) translate([ 0, 0, center ? -t / 2 : 0 ]) {
    cylinder(d = screw_m, h = leg_bot_h);
}
module screw_hole(center = false) translate([ 0, 0, center ? -t / 2 : 0 ]) {
    cylinder(d = screw_d, h = t);
    translate([ 0, 0, -0.01 ])
        cylinder(d1 = screw_e, d2 = screw_d, h = (screw_e - screw_d) / 2);
}
leg_bot_h = 4;
module leg_bot() {
    hex_d = 4;
    off_x = 8;
    off_y = 2.5;
    for (i = [ -0.5, 0.5 ])
        translate([ leg_bot_screw_r, i * screw_m, 0 ]) difference() {
            screw_mount();
            translate([ 0, 0, leg_bot_h - t ])
                Mscrew_hole(M2L4C, hole_offset = 10);
        }
    linear_extrude(height = leg_bot_h) difference() {
        leg_bot_base();
        for (i = [ -0.5, 0.5 ])
            translate([ leg_bot_screw_r, i * screw_m, 0 ]) circle(d = screw_m);
        intersection() {
            offset(-1) leg_bot_base();
            for (x = [0:off_x:bot_d / 2],
                y = [-bot_d / 2:hex_d + 1:bot_d / 2]) {
                translate([ x, y, 0 ]) circle(d = hex_d, $fn = 6);
                translate([ x + off_x / 2, y + off_y, 0 ])
                    circle(d = hex_d, $fn = 6);
            }
        }
    }
}

module rot_spring(w, range, step, r, t) {
    rotate([ 90, 0, 0 ]) linear_extrude(
        height = w, center = true) for (alpha = [range[0]:step:range[1]])
        hull() {
        translate(spring_f(alpha, 1.5 * r, r)) circle(d = t);
        translate(spring_f(alpha - step, 1.5 * r, r)) circle(d = t);
    }
}

tibia_bot_screw_r = sqrt(pow(b / 2 + t + screw_m / 2, 2) - pow(screw_m / 2, 2));

module tibia_bot_screws() for (alpha = [45:90:360], i = [ -0.5, 0.5 ])
    rotate([ 0, 0, alpha ]) translate([ tibia_bot_screw_r, i* screw_m, 0 ])
        children();

module bot_spring_screw_pos() {
    for (i = [ -0.5, 0.5 ]) translate([ 0, i * screw_m, 0 ]) {
            translate([ tibia_bot_screw_r, 0, 0 ]) children();
            translate([ leg_bot_screw_r, 0, -t ]) children();
        }
}

module bot_spring() difference() {
    union() {
        translate([ (tibia_bot_screw_r + leg_bot_screw_r) / 2, 0, 0 ])
            rot_spring(w = screw_m * 2, range = [ -45, 70 ], step = 1, r = 5.5,
                       t = 1);
        bot_spring_screw_pos() translate([ 0, 0, t ])
            cylinder(d = screw_m, h = t);
    }

    bot_spring_screw_pos() Mscrew_hole(M2L4F);
}

module tibia_bot_center() difference() {
    union() {
        cylinder(d = b + t * 4, h = t);
        intersection() {
            cylinder(d = b + t * 2, h = mid_h);
            union() {
                cylinder(d = b * 2, h = t);
                hull() for (z = [ bot_screw_z, 0 ]) translate([ 0, 0, z ])
                    rotate([ 90, 0, 0 ])
                        cylinder(d = screw_m, h = 100, center = true);
            }
        }
        tibia_bot_screws() cylinder(d = screw_m, h = t);
    }
    cylinder(d = ball_j_w, h = 100, center = true);
    translate([ 0, 0, bot_screw_z ]) {
        rotate([ 90, 0, 0 ]) cylinder(d = screw_d, h = 100, center = true);
        for (alpha = [-ball_angle_limit:10:ball_angle_limit])
            rotate([ 0, alpha, 0 ]) rotate([ 90, 0, 0 ]) hull() {
                cylinder(d = ball_j_d, h = ball_j_w, center = true);
                translate([ 0, 50, 0 ])
                    cylinder(d = ball_j_d, h = ball_j_w, center = true);
            }
    }
    tibia_bot_screws() Mscrew_hole(M2L4C);
}

module tibia_bot(expand = 0) {
    for (alpha = [45:90:360]) rotate([ 0, 0, alpha ]) {
            translate([ expand, 0, -expand ]) finalize("orange") bot_spring();
            translate([ 2 * expand, 0, -leg_bot_h - expand * 2 ])
                finalize("blue") leg_bot();
            translate([ 0, 0, -3 * expand ]) bot_spring_screw_pos()
                Mscrew(M2L4C);
        }
    translate([ 0, 0, bot_screw_z ]) rotate([ 90, 0, 0 ]) {
        translate([ 0, 0, -b / 2 - t - expand * 4 ]) Mscrew(M2L12C);
        translate([ 0, 0, b / 2 + t + expand * 4 ]) Mnut(M2N);
    }
    finalize("blue") tibia_bot_center();
}

module tibia(tipOffset, expand = 0) {
    tibia_servo_pos() LX15D();
    translate([ 2 * expand, 0, 0 ]) tibia_servo_pos() {
        LX15D_U(class = T24, expand = expand);
        translate(LX15D_U_offset + [ -Mnut_h(M2N) - Tt(T24) - expand, 0, 0 ])
            rotate([ 0, 90, 0 ]) tile_screw_pos(T24) {
            Mscrew(M2L8C);
            translate([ 0, 0, Tt(T24) + Mnut_h(M2N) + Tt(T24) + 3 * expand ])
                Mnut(M2N);
        }
    }
    translate(LX15D_U_offset) rotate([ 0, 90, 0 ])
        translate([ 0, 0, 5 * expand ]) {
        translate([ 0, 0, -2 * expand ]) tile_H(T24, middle_tile_h);
        tibia_pyramid_screw_pos(tipOffset) {
            translate([ 0, 0, -1.5 * expand ]) Mscrew(M2L8C);
        }
        tibia_pyramid_screw_pos(tipOffset, ignore_middle = true) {
            translate([ 0, 0, Tt(T24) + 3 * expand ]) Mnut(M2N);
        }
    }
    translate([ 5 * expand, 0, 0 ]) finalize("orange") tibia_pyramid(tipOffset);
    translate(tipOffset) {
        translate([ 7 * expand - bot_screw_z, 0, 0 ]) rotate([ 0, -90, 0 ])
            ball_joint();
        translate([ 8 * expand - bot_screw_z, 0, 0 ]) rotate([ 0, 25 - 90, 0 ])
            translate([ 0, 0, -bot_screw_z ]) {
            tibia_bot(expand = expand);
        }
    }
}

module tibia_collision(tipOffset) {
    dim = [ 64, 40, 26 ];
    dim2 = [ tipOffset[0], 6, 6 ];
    off = [ 20, 0, 0 ];

    hull() {
        translate(off) cube(dim, center = true);

        translate([ dim2[0] / 2, 0, 0 ]) cube(dim2, center = true);
    }
}
//tibia_bot(expand=10);
render_part = "nothing";
if(render_part == "tibia_bot_center"){
    $fn = 128;
    tibia_bot_center();
} else if (render_part == "bot_spring"){
    $fn = 128;
    bot_spring();
} else if (render_part == "leg_bot"){
    $fn = 128;
    leg_bot();
} 
